﻿using Plugin.Permissions;
using Plugin.Permissions.Abstractions;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;

using Xamarin.Forms;
using Xamarin.Forms.Xaml;

namespace App2
{
    public interface IDataStore<T>
    {
        Task<bool> AddItemAsync(T item);
        Task<bool> UpdateItemAsync(T item);
        Task<bool> DeleteItemAsync(string id);
        Task<T> GetItemAsync(string id);
        Task<IEnumerable<T>> GetItemsAsync(bool forceRefresh = false, int lastIndex = 0);
    }

    public class MockDataStore : IDataStore<Item>
    {
        readonly List<Item> items;

        public MockDataStore()
        {
            items = new List<Item>()
            {
                new Item { Id = Guid.NewGuid().ToString(), Text = "First item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Second item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Third item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Fourth item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Fifth item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Sixth item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Seventh item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Eigth item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Nineth item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Tenth item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Eleventh item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Twelveth item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Thirteenth item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Fourteenth item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Fifteenth item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Sixteenth item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Seventeenth item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Eighteenth item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Nineteenth item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Twentieth item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Twentyfirst item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Twentysecond item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Twentythird item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Twentyfourth item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Twentyfifth item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Twentysixth item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Twentyseventh item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Twentyeight item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Twentynineth item", Description="This is an item description." },
                new Item { Id = Guid.NewGuid().ToString(), Text = "Thirtieth item", Description="This is an item description." }
            };
        }

        public async Task<bool> AddItemAsync(Item item)
        {
            items.Add(item);

            return await Task.FromResult(true);
        }

        public async Task<bool> UpdateItemAsync(Item item)
        {
            var oldItem = items.Where((Item arg) => arg.Id == item.Id).FirstOrDefault();
            items.Remove(oldItem);
            items.Add(item);

            return await Task.FromResult(true);
        }

        public async Task<bool> DeleteItemAsync(string id)
        {
            var oldItem = items.Where((Item arg) => arg.Id == id).FirstOrDefault();
            items.Remove(oldItem);

            return await Task.FromResult(true);
        }

        public async Task<Item> GetItemAsync(string id)
        {
            return await Task.FromResult(items.FirstOrDefault(s => s.Id == id));
        }

        public async Task<IEnumerable<Item>> GetItemsAsync(bool forceRefresh = false, int lastIndex = 0)
        {
            await Task.Delay(TimeSpan.FromSeconds(0.01));
            int numberOfItemsPerPage = 15;
            return await Task.FromResult(items.Skip(lastIndex).Take(numberOfItemsPerPage));
        }
    }

    public class Item
    {
        public string Id { get; set; }
        public string Text { get; set; }
        public string Description { get; set; }
    }
    public class BaseViewModel : INotifyPropertyChanged
    {
        public IDataStore<Item> DataStore => DependencyService.Get<IDataStore<Item>>();

        bool isBusy = false;
        public bool IsBusy
        {
            get { return isBusy; }
            set { SetProperty(ref isBusy, value); }
        }

        string title = string.Empty;
        public string Title
        {
            get { return title; }
            set { SetProperty(ref title, value); }
        }

        protected bool SetProperty<T>(ref T backingStore, T value,
            [CallerMemberName]string propertyName = "",
            Action onChanged = null)
        {
            if (EqualityComparer<T>.Default.Equals(backingStore, value))
                return false;

            backingStore = value;
            onChanged?.Invoke();
            OnPropertyChanged(propertyName);
            return true;
        }

        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;
        protected void OnPropertyChanged([CallerMemberName] string propertyName = "")
        {
            var changed = PropertyChanged;
            if (changed == null)
                return;

            changed.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion
    }

    public class ItemsViewModel : BaseViewModel
    {
        public ObservableCollection<Item> Items { get; set; }
        public Command LoadItemsCommand { get; set; }
        public Command ItemTresholdReachedCommand { get; set; }
        public Command RefreshItemsCommand { get; set; }
        public const string ScrollToPreviousLastItem = "Scroll_ToPrevious";
        private int _itemTreshold;
        private bool _isRefreshing;

        public bool IsRefreshing
        {
            get { return _isRefreshing; }
            set { SetProperty(ref _isRefreshing, value); }
        }

        public int ItemTreshold
        {
            get { return _itemTreshold; }
            set { SetProperty(ref _itemTreshold, value); }
        }

        public ItemsViewModel()
        {
            ItemTreshold = 1;
            Title = "Browse";
            Items = new ObservableCollection<Item>();
            LoadItemsCommand = new Command(async () => await ExecuteLoadItemsCommand());
            ItemTresholdReachedCommand = new Command(async () => await ItemsTresholdReached());
            RefreshItemsCommand = new Command(async () =>
            {
                await ExecuteLoadItemsCommand();
                IsRefreshing = false;
            });
        }

        async Task ItemsTresholdReached()
        {
            if (IsBusy)
                return;

            IsBusy = true;

            try
            {
                var items = await DataStore.GetItemsAsync(true, Items.Count);

                
                foreach (var item in items)
                {
                    Items.Add(item);
                }



                Debug.WriteLine($"{items.Count()} {Items.Count} ");
                if (items.Count() == 0)
                {
                    ItemTreshold = 1;

                    LoadItemsCommand.Execute(null);

                   // Items.Clear();

                    //var previousLastItem = Items[0];
                    // MessagingCenter.Send<object, Item>(this, ScrollToPreviousLastItem, previousLastItem);
                    return;
                }
                
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex);
            }
            finally
            {
                IsBusy = false;
            }
        }

        async Task ExecuteLoadItemsCommand()
        {


            IsBusy = true;

            try
            {
                ItemTreshold = 1;
                Items.Clear();
                var items = await DataStore.GetItemsAsync(true);
                foreach (var item in items)
                {
                    Items.Add(item);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex);
            }
            finally
            {
                IsBusy = false;
            }
        }
    }


    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class Page1 : ContentPage
    {

        ItemsViewModel viewModel;
        public Page1()
        {
            InitializeComponent();

            BindingContext = viewModel = new ItemsViewModel();
        }
        protected override void OnAppearing()
        {
            base.OnAppearing();

            if (viewModel.Items.Count == 0)
                viewModel.LoadItemsCommand.Execute(null);

            MessagingCenter.Subscribe<object, Item>(this, ItemsViewModel.ScrollToPreviousLastItem, (sender, item) =>
            {
                ItemsCollectionView.ScrollTo(item, ScrollToPosition.End);
            });
        }
    }
}